class Json2html {
    /**
     * @param json one ou list of node data
     * @param option formating options
     */
    constructor(json, option = {}) {
        this.json = json;
        this.options = {
            spaceType: 'space',
            spaceLength: 4,
            spaceBase: 0,
            // maxLenght: 0,
            attrPosition: 'alignFirstAttr',
            type: 'html',
            formatting: 'multiline',
            indent: true,
            xmlDefaultTag: 'span',
            noContentTags: [
                'area',
                'base',
                'br',
                'col',
                'command',
                'embed',
                'hr',
                'img',
                'input',
                'keygen',
                'link',
                'meta',
                'param',
                'source',
                'track',
                'wbr'
            ],
            removeOptionalEndTags: false,
            optionalEndTags: [
                'colgroup',
                'dd',
                'dt',
                'li',
                'optgroup',
                'option',
                'p',
                'rb',
                'rt',
                'rtc',
                'rp',
                'td',
                'th',
                'thead',
                'tbody',
                'tfoot',
                'tr'
            ]
        };
        Object.assign(this.options, option);
    }
    /**
     * rendering in string
     */
    toString() {
        let html = '';
        if (!Array.isArray(this.json)) {
            html = `${this._getSpacing(0)}${this._generate(0, this.json)}`;
        }
        else {
            this.json.forEach((element, index) => {
                html += `${index > 0 ? '\n' : ''}${this._getSpacing(0)}${this._generate(0, element)}`;
            });
        }
        return html;
    }
    /**
     * generation in string for a node (tag with attributes and content)
     * @param lvl node level
     * @param json node data
     * @returns render of node
     */
    _generate(lvl, json, inline = false) {
        const hasContent = !this.options.noContentTags.includes(json.tag.toLowerCase());
        const xmlAutoClose = (!hasContent || json.autoclose) && this._modeXML() ? '/' : '';
        let string = `<${json.tag}${this._generateAttrs(lvl, json, inline)}${xmlAutoClose}>`;
        if (hasContent && !json.autoclose) {
            let tagcontent = this._generateBody(lvl, json, inline || json.inline);
            if (tagcontent && this._hasMultiline() && !(inline || json.inline)) {
                tagcontent = `${tagcontent}\n${this._getSpacing(lvl)}`;
            }
            string += tagcontent;
            if (!this.options.removeOptionalEndTags ||
                this._modeXML() ||
                this.options.removeOptionalEndTags && !this.options.optionalEndTags.includes(json.tag.toLowerCase())) {
                string += `</${json.tag}>`;
            }
        }
        return string;
    }
    /**
     * attributes list generation
     * @param lvl level node
     * @param json node data
     * @returns attributes tag
     */
    _generateAttrs(lvl, json, inline) {
        let string = '';
        const attrs = json.attrs;
        if (attrs && Object.keys(attrs).length) {
            for (const id in attrs) {
                if (attrs[id] !== undefined) {
                    let attr = '';
                    switch (!inline ? this.options.attrPosition : 'inline') {
                        case 'inline':
                            attr += ' ';
                            break;
                        case 'space':
                            attr += string && this.options.indent && this._hasMultiline()
                                ? `\n${this._getSpacing(lvl + 1)}`
                                : ' ';
                            break;
                        case 'alignTag':
                            attr += string && this.options.indent && this._hasMultiline()
                                ? `\n${this._getSpacing(lvl, 1)}`
                                : ' ';
                            break;
                        case 'alignFirstAttr':
                            attr += string && this.options.indent && this._hasMultiline()
                                ? `\n${this._getSpacing(lvl, json.tag.length + 2)}`
                                : ' ';
                            break;
                    }
                    attr += `${id}${attrs[id] !== null || attrs[id] ? `="${attrs[id]}"` : ''}`;
                    string += attr;
                }
            }
        }
        return string;
    }
    /**
     * tag body generation
     * @param lvl level node
     * @param json node data
     * @returns render of body
     */
    _generateBody(lvl, json, inline) {
        let string = '';
        if (json.body) {
            if (!Array.isArray(json.body)) {
                string += this._generateBodyElement(lvl, json.body, true, inline);
            }
            else {
                json.body.forEach(element => {
                    string += this._generateBodyElement(lvl, element, false, inline);
                });
            }
        }
        return string;
    }
    /**
     * tag body generation for one node
     * @param lvl level node
     * @param element node data or string
     * @param onlyOne body this an unique node
     * @returns render of body
     */
    _generateBodyElement(lvl, element, onlyOne, inline = false) {
        let string = '';
        if (this._hasMultiline() && !inline) {
            string += `\n${this._getSpacing(lvl + 1)}`;
        }
        // in XML mode, for generate a valid XML structure
        if (!onlyOne && this._modeXML() && typeof element === 'string') {
            element = {
                tag: this.options.xmlDefaultTag,
                body: element
            };
        }
        string += typeof element === 'string'
            ? element
            : this._generate(lvl + 1, element, inline);
        return string;
    }
    /**
     * if multiline rendering
     */
    _hasMultiline() {
        return this.options.formatting === 'multiline';
    }
    /**
     * if multiline rendering
     */
    _modeXML() {
        return this.options.type === 'xml';
    }
    /**
     * calculating the number of spaces for indentation
     * @param lvl level
     * @param addition space of space (only space, no tabulation)
     * @returns space
     */
    _getSpacing(lvl, addition = 0) {
        return this.options.indent
            ? (this.options.spaceType === 'space' ? ' ' : '\t')
                .repeat((lvl + +this.options.spaceBase) * +this.options.spaceLength) + ' '.repeat(addition)
            : '';
    }
}

/*
 * Public API Surface of json2html
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Json2html };
//# sourceMappingURL=json2html-lib.mjs.map
